<?php
 
class SynoDebridLink {

    const API_URL = 'https://debrid-link.fr/api/v2';
    const OAUTH_API_URL = 'https://debrid-link.com/api/oauth';
    const OAUTH_CLIENT_ID = 'debrid-link-synology';
    const OAUTH_SCOPE = 'post.downloader get.account';

    private $oauthSessionFile = '/tmp/debridlink.cookie';
    private $oauthSession;

    private $invalidCredentialsFile = '/tmp/debridlink.error';

    private $url;
    private $username;
    private $password;

    public function __construct($url, $username, $password, $hostInfo){
        $this->username = $username;
        $this->password = $password;
        $this->url = $url;
    }

    public function Verify($clearCookie){

        if($clearCookie){
            $this->cleanOauthSession();
        }

        $res = $this->api('GET', '/account/infos');

        if(isset($res['value']['accountType'])){
            return $res['value']['accountType'] > 0 ? USER_IS_PREMIUM : USER_IS_FREE;
        }
        
        return LOGIN_FAIL;

    }

    public function GetDownloadInfo() {

        $res = $this->api('POST', '/downloader/add', array('url' => $this->url));

        if($res['success']){
            return array(
                DOWNLOAD_URL => $res['value']['downloadUrl'],
                DOWNLOAD_FILENAME => $res['value']['name'],
                DOWNLOAD_ISPARALLELDOWNLOAD => ($res['value']['chunk'] > 1)
            );
        }elseif($res['error'] == 'fileNotFound'){
            return ERR_FILE_NO_EXIST;
        }elseif(isset($res['upgradeUrl'])){
            return ERR_REQUIRED_PREMIUM;
        }
        
        return false;

    }

    private function login(){

        if(!$this->username || !$this->password){
            return false;
        }

        // Avoid blocking client IP by flooding login endpoint with invalid credentials
        if($this->isInvalidCredentials($this->username, $this->password)){
            return false;
        }

        $res = $this->oauth('POST', '/token', array(
            'client_id' => self::OAUTH_CLIENT_ID,
            'username' => $this->username,
            'password' => $this->password,
            'grant_type' => 'password',
            'scope' => self::OAUTH_SCOPE
        ));

        if(isset($res['access_token'])){
            $this->setOauthSession($res);
            return true;
        }

        // Invalid username or password / account locked
        if($res['error'] == 'invalid_request' || $res['error'] == 'accountLocked'){
            $this->setInvalidCredentials($this->username, $this->password);
        }

        return false;

    }

    private function api($method, $path, $data = array(), $retry = false){

        $oauthSession = $this->getOauthSession();

        // Get oauth if required then retry
        if(!isset($oauthSession['access_token']) && !isset($oauthSession['refresh_token'])){
            if(!$retry && $this->login()){
                return $this->api($method, $path, $data, true);
            }
            return array('success' => false, 'error' => 'badToken');
        }

        $res = $this->request(array(
            'method' => $method,
            'url' => self::API_URL . $path,
            'headers' => array(
                'authorization' => 'Bearer '. $oauthSession['access_token']
            ),
            'data' => $data
        ));

        // Refresh access_token then retry
        if(!$res['success'] && $res['error'] == 'badToken' && !$retry && isset($oauthSession['refresh_token'])){

            $refreshToken = $this->oauth('POST', '/token', array(
                'client_id' => self::OAUTH_CLIENT_ID,
                'refresh_token' => $oauthSession['refresh_token'],
                'grant_type' => 'refresh_token'
            ));

            if(isset($refreshToken['access_token'])){
                $this->setOauthSession($refreshToken);
                return $this->api($method, $path, $data, true);
            }

            $this->cleanOauthSession();

        }

        return $res;

    }

    private function oauth($method, $path, $data = array()){
        return $this->request(array(
            'method' => $method,
            'url' => self::OAUTH_API_URL . $path,
            'data' => $data
        ));
    }

    private function request($opts = array()){

        $opts = array_merge(array(
            'method' => 'GET',
            'url' => false,
            'headers' => array(),
            'data' => array()
        ), $opts);

        $opts['headers']['user-agent'] = 'Debrid-Link Synology';

        if($opts['method'] == 'POST'){
            $opts['headers']['content-type'] = 'application/json';
        }

        if(!isset($opts['headers']['accept'])){
            $opts['headers']['accept'] = 'application/json';
        }

        $headers = array();
        foreach($opts['headers'] as $key => $value){
            $headers[] = $key .': '. $value;
        }

        $this->debug($opts['method'] . ' : '. $opts['url']);

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $opts['url']);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $opts['method']);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_USERAGENT,  'Debrid-Link Synology');
        curl_setopt($ch, CURLOPT_HEADER, false);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

        if($opts['method'] == 'POST'){
            curl_setopt($ch, CURLOPT_POST, true);
            curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($opts['data']));
        }

        $res = curl_exec($ch);
        curl_close($ch);

        $res = json_decode($res, true);

        if(!$res){
            $res = array('success' => false, 'error' => 'networkError');
        }

        $this->debug($res);

        return $res;

    }

    private function getOauthSession(){
        if(is_null($this->oauthSession)){
            $oauthSession = @file_get_contents($this->oauthSessionFile);
            $this->oauthSession = $oauthSession ? json_decode($oauthSession, true) : array();
        }
        return $this->oauthSession;
    }

    private function cleanOauthSession(){
        $this->oauthSession = array();
        @unlink($this->oauthSessionFile);
    }

    private function setOauthSession($oauthSession){
        if(isset($oauthSession['access_token'])){
            $this->oauthSession['access_token'] = $oauthSession['access_token'];
        }
        if(isset($oauthSession['refresh_token'])){
            $this->oauthSession['refresh_token'] = $oauthSession['refresh_token'];
        }
        if(isset($oauthSession['expires_in'])){
            $this->oauthSession['expires_in'] = $oauthSession['expires_in'];
            $this->oauthSession['expires_at'] = time() + $oauthSession['expires_in'];
        }
        file_put_contents($this->oauthSessionFile, json_encode($this->oauthSession));
    }

    private function isInvalidCredentials($username, $password){
        $invalidCredentials = @file_get_contents($this->invalidCredentialsFile);
        $invalidCredentials = $invalidCredentials ? json_decode($invalidCredentials, true) : array();
        return (
            $invalidCredentials 
            && $invalidCredentials['hash'] == sha1($username . $password) 
            && $invalidCredentials['time'] + 3600 > time()
        );
    }

    private function setInvalidCredentials($username, $password){
        $invalidCredentials = array('hash' => sha1($username . $password), 'time' => time());
        file_put_contents($this->invalidCredentialsFile, json_encode($invalidCredentials));
    }

    private function debug($message){
        error_log(is_array($message) ? json_encode($message) : $message);
    }

}